@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of the python-chess library.
-# Copyright (C) 2016 Niklas Fiekas <niklas.fiekas@backscattering.de>
+# Copyright (C) 2016-2017 Niklas Fiekas <niklas.fiekas@backscattering.de>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,16 +44,13 @@ class SuicideBoard(chess.Board):
     def _attacked_for_king(self, path):
         return False
 
-    def _ep_skewered(self, capturer_mask):
-        return False
-
     def is_check(self):
         return False
 
     def is_into_check(self, move):
         return False
 
-    def was_into_check(self, move):
+    def was_into_check(self):
         return False
 
     def _material_balance(self):
@@ -177,7 +174,7 @@ class GiveawayBoard(SuicideBoard):
     pawnless_tbz_magic = [0xe4, 0xcf, 0xe7, 0x23]
 
     def __init__(self, fen=starting_fen, chess960=False):
-        super(GiveawayBoard, self).__init__(fen, chess960)
+        super(GiveawayBoard, self).__init__(fen, chess960=chess960)
 
     def reset(self):
         super(GiveawayBoard, self).reset()
@@ -221,7 +218,7 @@ class AtomicBoard(chess.Board):
         if self.pawns or self.queens:
             return False
 
-        if chess.popcount(self.KNIGHT | self.BISHOP | self.ROOK) == 1:
+        if chess.popcount(self.knights | self.bishops | self.rooks) == 1:
             return True
 
         # Only knights.
@@ -298,7 +295,7 @@ class AtomicBoard(chess.Board):
                 yield move
 
     def status(self):
-        status = super(SuicideBoard, self).status()
+        status = super(AtomicBoard, self).status()
         status &= ~chess.STATUS_OPPOSITE_CHECK
         if self.turn == chess.WHITE:
             status &= ~chess.STATUS_NO_WHITE_KING
@@ -340,7 +337,7 @@ class RacingKingsBoard(chess.Board):
     tbw_magic = tbz_magic = None
 
     def __init__(self, fen=starting_fen, chess960=False):
-        super(RacingKingsBoard, self).__init__(fen, chess960)
+        super(RacingKingsBoard, self).__init__(fen, chess960=chess960)
 
     def reset(self):
         self.set_fen(type(self).starting_fen)
@@ -421,7 +418,7 @@ class HordeBoard(chess.Board):
     tbw_magic = tbz_magic = None
 
     def __init__(self, fen=starting_fen, chess960=False):
-        super(HordeBoard, self).__init__(fen, chess960)
+        super(HordeBoard, self).__init__(fen, chess960=chess960)
 
     def reset(self):
         self.set_fen(type(self).starting_fen)
@@ -475,7 +472,7 @@ class ThreeCheckBoard(chess.Board):
 
     def __init__(self, fen=starting_fen, chess960=False):
         self.remaining_checks = [3, 3]
-        super(ThreeCheckBoard, self).__init__(fen, chess960)
+        super(ThreeCheckBoard, self).__init__(fen, chess960=chess960)
 
     def reset_board(self):
         super(ThreeCheckBoard, self).reset_board()
@@ -543,7 +540,7 @@ class ThreeCheckBoard(chess.Board):
             raise ValueError("three-check fen should consist of 7 parts: {0}".format(repr(fen)))
 
         # Extract check part.
-        if "+" == parts[6][0]:
+        if parts[6][0] == "+":
             check_part = parts.pop()[1:]
             try:
                 w, b = check_part.split("+", 1)
@@ -634,7 +631,7 @@ class CrazyhouseBoard(chess.Board):
 
     def __init__(self, fen=starting_fen, chess960=False):
         self.pockets = [CrazyhousePocket(), CrazyhousePocket()]
-        super(CrazyhouseBoard, self).__init__(fen, chess960)
+        super(CrazyhouseBoard, self).__init__(fen, chess960=chess960)
 
     def reset_board(self):
         super(CrazyhouseBoard, self).reset_board()
